Package: rvinecopulib
Type: Package
Title: High Performance Algorithms for Vine Copula Modeling
Version: 0.7.3.1.0
Authors@R: c(
        person("Thomas", "Nagler",, "info@vinecopulib.org", role = c("aut", "cre")),
        person("Thibault", "Vatter",, "info@vinecopulib.org", role = c("aut"))
        )
Description: Provides an interface to 'vinecopulib', a C++ library for vine 
 copula modeling. The 'rvinecopulib' package implements the core features of the
 popular 'VineCopula' package, in particular inference algorithms for both vine 
 copula and bivariate copula models. Advantages over 'VineCopula' are a sleeker 
 and more modern API, improved performances, especially in high dimensions, 
 nonparametric and multi-parameter families, and the ability to model discrete 
 variables. The 'rvinecopulib' package includes 'vinecopulib' as header-only 
 C++ library (currently version 0.7.2). Thus users do not need to install 
 'vinecopulib' itself in order to use 'rvinecopulib'. Since their initial 
 releases, 'vinecopulib' is licensed under the MIT License, and 'rvinecopulib' 
 is licensed under the GNU GPL version 3.
License: GPL-3 | file LICENSE
Encoding: UTF-8
NeedsCompilation: yes
Depends: R (>= 3.0.2)
Imports: assertthat, graphics, grDevices, kde1d (>= 1.1.0), lattice,
        Rcpp (>= 0.12.12), stats, utils
Suggests: igraph, ggplot2, ggraph, testthat
LinkingTo: BH, Rcpp, RcppEigen, RcppThread (>= 2.1.2), wdm (>= 0.2.6),
        kde1d (>= 1.1.0)
BugReports: https://github.com/vinecopulib/rvinecopulib/issues
URL: https://vinecopulib.github.io/rvinecopulib/
RoxygenNote: 7.3.2
Packaged: 2025-06-13 11:05:32 UTC; n5
Author: Thomas Nagler [aut, cre],
  Thibault Vatter [aut]
Maintainer: Thomas Nagler <info@vinecopulib.org>
Repository: CRAN
Date/Publication: 2025-06-13 12:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 03:12:13 UTC; unix
Archs: rvinecopulib.so.dSYM
