# Generated by roxygen2: do not edit by hand

S3method(as_rvine_matrix,list)
S3method(as_rvine_matrix,matrix)
S3method(as_rvine_matrix,rvine_matrix)
S3method(as_rvine_matrix,rvine_structure)
S3method(as_rvine_structure,list)
S3method(as_rvine_structure,matrix)
S3method(as_rvine_structure,rvine_matrix)
S3method(as_rvine_structure,rvine_structure)
S3method(coef,bicop_dist)
S3method(contour,bicop)
S3method(contour,bicop_dist)
S3method(contour,vinecop)
S3method(contour,vinecop_dist)
S3method(dim,rvine_matrix)
S3method(dim,rvine_structure)
S3method(dim,vine_dist)
S3method(dim,vinecop_dist)
S3method(fitted,bicop)
S3method(fitted,vine)
S3method(fitted,vinecop)
S3method(logLik,bicop)
S3method(logLik,vine)
S3method(logLik,vinecop)
S3method(plot,bicop)
S3method(plot,bicop_dist)
S3method(plot,rvine_matrix)
S3method(plot,rvine_structure)
S3method(plot,vinecop)
S3method(plot,vinecop_dist)
S3method(predict,bicop_dist)
S3method(predict,vine)
S3method(predict,vinecop)
S3method(print,bicop)
S3method(print,bicop_dist)
S3method(print,rvine_list)
S3method(print,rvine_matrix)
S3method(print,rvine_structure)
S3method(print,summary_df)
S3method(print,vine)
S3method(print,vine_dist)
S3method(print,vinecop)
S3method(print,vinecop_dist)
S3method(summary,bicop)
S3method(summary,bicop_dist)
S3method(summary,vine)
S3method(summary,vine_dist)
S3method(summary,vinecop)
S3method(summary,vinecop_dist)
S3method(truncate_model,rvine_matrix)
S3method(truncate_model,rvine_structure)
S3method(truncate_model,vine_dist)
S3method(truncate_model,vinecop_dist)
export(as.bicop)
export(as_rvine_matrix)
export(as_rvine_structure)
export(bicop)
export(bicop_dist)
export(cvine_structure)
export(dbicop)
export(dvine)
export(dvine_structure)
export(dvinecop)
export(emp_cdf)
export(get_all_families)
export(get_all_ktaus)
export(get_all_pair_copulas)
export(get_all_parameters)
export(get_family)
export(get_ktau)
export(get_matrix)
export(get_pair_copula)
export(get_parameters)
export(get_structure)
export(hbicop)
export(inverse_rosenblatt)
export(is.rvine_matrix)
export(is.rvine_structure)
export(ktau_to_par)
export(mBICV)
export(pairs_copula_data)
export(par_to_ktau)
export(pbicop)
export(pseudo_obs)
export(pvine)
export(pvinecop)
export(rbicop)
export(rosenblatt)
export(rvine)
export(rvine_matrix)
export(rvine_matrix_sim)
export(rvine_structure)
export(rvine_structure_sim)
export(rvinecop)
export(truncate_model)
export(vine)
export(vine_dist)
export(vinecop)
export(vinecop_dist)
importFrom(Rcpp,evalCpp)
importFrom(assertthat,"on_failure<-")
importFrom(assertthat,assert_that)
importFrom(assertthat,is.count)
importFrom(assertthat,is.flag)
importFrom(assertthat,is.number)
importFrom(assertthat,is.scalar)
importFrom(assertthat,is.string)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,box)
importFrom(graphics,contour)
importFrom(graphics,hist.default)
importFrom(graphics,pairs.default)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,points.default)
importFrom(graphics,polygon)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,text.default)
importFrom(kde1d,dkde1d)
importFrom(kde1d,pkde1d)
importFrom(kde1d,qkde1d)
importFrom(lattice,wireframe)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,dcauchy)
importFrom(stats,dchisq)
importFrom(stats,dexp)
importFrom(stats,df)
importFrom(stats,dgamma)
importFrom(stats,dlnorm)
importFrom(stats,dnorm)
importFrom(stats,dt)
importFrom(stats,dunif)
importFrom(stats,dweibull)
importFrom(stats,ecdf)
importFrom(stats,logLik)
importFrom(stats,model.matrix)
importFrom(stats,pbeta)
importFrom(stats,pcauchy)
importFrom(stats,pchisq)
importFrom(stats,pexp)
importFrom(stats,pf)
importFrom(stats,pgamma)
importFrom(stats,plnorm)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,punif)
importFrom(stats,pweibull)
importFrom(stats,qbeta)
importFrom(stats,qcauchy)
importFrom(stats,qchisq)
importFrom(stats,qexp)
importFrom(stats,qf)
importFrom(stats,qgamma)
importFrom(stats,qlnorm)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,qunif)
importFrom(stats,qweibull)
importFrom(stats,runif)
importFrom(utils,capture.output)
importFrom(utils,modifyList)
importFrom(utils,write.table)
useDynLib(rvinecopulib, .registration = TRUE)
