# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

pseudo_obs_cpp <- function(x, ties_method) {
    .Call(`_rvinecopulib_pseudo_obs_cpp`, x, ties_method)
}

bicop_check_cpp <- function(bicop_r) {
    invisible(.Call(`_rvinecopulib_bicop_check_cpp`, bicop_r))
}

bicop_select_cpp <- function(data, family_set, par_method, nonpar_method, mult, selcrit, weights, psi0, presel, num_threads, var_types) {
    .Call(`_rvinecopulib_bicop_select_cpp`, data, family_set, par_method, nonpar_method, mult, selcrit, weights, psi0, presel, num_threads, var_types)
}

bicop_pdf_cpp <- function(u, bicop_r) {
    .Call(`_rvinecopulib_bicop_pdf_cpp`, u, bicop_r)
}

bicop_cdf_cpp <- function(u, bicop_r) {
    .Call(`_rvinecopulib_bicop_cdf_cpp`, u, bicop_r)
}

bicop_hfunc1_cpp <- function(u, bicop_r) {
    .Call(`_rvinecopulib_bicop_hfunc1_cpp`, u, bicop_r)
}

bicop_hfunc2_cpp <- function(u, bicop_r) {
    .Call(`_rvinecopulib_bicop_hfunc2_cpp`, u, bicop_r)
}

bicop_hinv1_cpp <- function(u, bicop_r) {
    .Call(`_rvinecopulib_bicop_hinv1_cpp`, u, bicop_r)
}

bicop_hinv2_cpp <- function(u, bicop_r) {
    .Call(`_rvinecopulib_bicop_hinv2_cpp`, u, bicop_r)
}

bicop_sim_cpp <- function(bicop_r, n, qrng, seeds) {
    .Call(`_rvinecopulib_bicop_sim_cpp`, bicop_r, n, qrng, seeds)
}

bicop_loglik_cpp <- function(u, bicop_r) {
    .Call(`_rvinecopulib_bicop_loglik_cpp`, u, bicop_r)
}

bicop_par_to_tau_cpp <- function(bicop_r) {
    .Call(`_rvinecopulib_bicop_par_to_tau_cpp`, bicop_r)
}

bicop_tau_to_par_cpp <- function(bicop_r, tau) {
    .Call(`_rvinecopulib_bicop_tau_to_par_cpp`, bicop_r, tau)
}

rvine_structure_cpp <- function(rvine_structure_r, check, is_natural_order) {
    .Call(`_rvinecopulib_rvine_structure_cpp`, rvine_structure_r, check, is_natural_order)
}

rvine_structure_check_cpp <- function(rvine_struct, is_natural_order) {
    invisible(.Call(`_rvinecopulib_rvine_structure_check_cpp`, rvine_struct, is_natural_order))
}

rvine_structure_sim_cpp <- function(d, natural_order, seeds) {
    .Call(`_rvinecopulib_rvine_structure_sim_cpp`, d, natural_order, seeds)
}

rvine_matrix_check_cpp <- function(matrix) {
    invisible(.Call(`_rvinecopulib_rvine_matrix_check_cpp`, matrix))
}

vinecop_check_cpp <- function(vinecop_r) {
    invisible(.Call(`_rvinecopulib_vinecop_check_cpp`, vinecop_r))
}

vinecop_inverse_rosenblatt_cpp <- function(U, vinecop_r, cores) {
    .Call(`_rvinecopulib_vinecop_inverse_rosenblatt_cpp`, U, vinecop_r, cores)
}

vinecop_rosenblatt_cpp <- function(U, vinecop_r, cores) {
    .Call(`_rvinecopulib_vinecop_rosenblatt_cpp`, U, vinecop_r, cores)
}

vinecop_sim_cpp <- function(vinecop_r, n, qrng, cores, seeds) {
    .Call(`_rvinecopulib_vinecop_sim_cpp`, vinecop_r, n, qrng, cores, seeds)
}

vinecop_pdf_cpp <- function(u, vinecop_r, cores) {
    .Call(`_rvinecopulib_vinecop_pdf_cpp`, u, vinecop_r, cores)
}

vinecop_cdf_cpp <- function(u, vinecop_r, N, cores, seeds) {
    .Call(`_rvinecopulib_vinecop_cdf_cpp`, u, vinecop_r, N, cores, seeds)
}

vinecop_loglik_cpp <- function(u, vinecop_r, cores) {
    .Call(`_rvinecopulib_vinecop_loglik_cpp`, u, vinecop_r, cores)
}

vinecop_mbicv_cpp <- function(u, vinecop_r, psi0, cores) {
    .Call(`_rvinecopulib_vinecop_mbicv_cpp`, u, vinecop_r, psi0, cores)
}

vinecop_select_cpp <- function(data, structure, family_set, par_method, nonpar_method, mult, truncation_level, tree_criterion, threshold, selection_criterion, weights, psi0, select_truncation_level, select_threshold, preselect_families, show_trace, num_threads, var_types) {
    .Call(`_rvinecopulib_vinecop_select_cpp`, data, structure, family_set, par_method, nonpar_method, mult, truncation_level, tree_criterion, threshold, selection_criterion, weights, psi0, select_truncation_level, select_threshold, preselect_families, show_trace, num_threads, var_types)
}

fit_margins_cpp <- function(data, nlevels, mult, xmin, xmax, bw, deg, weights, num_threads) {
    .Call(`_rvinecopulib_fit_margins_cpp`, data, nlevels, mult, xmin, xmax, bw, deg, weights, num_threads)
}

