% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairs_copula_data.R
\name{pairs_copula_data}
\alias{pairs_copula_data}
\title{Exploratory pairs plot for copula data}
\usage{
pairs_copula_data(data, ...)
}
\arguments{
\item{data}{the data (must lie in the unit hypercube).}

\item{\dots}{other parameters passed to \code{pairs.default()},
\code{contour.bicop()}, \code{points.default()}, \code{hist.default()}, or \code{bicop()}.}
}
\description{
This function provides pair plots for copula data. It shows
bivariate contour plots on the lower panel, scatter plots and
correlations on the upper panel and histograms on the diagonal panel.
}
\examples{
u <- replicate(3, runif(100))
pairs_copula_data(u)
}
